/*
  fblogo.c
  Generate linux_logo.h header file for use with the framebuffer boot logo
  Written by Daniel Vedy <daniel@kongsberg.online.no> 1999
  Released under the GPL
*/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <magick/api.h>
#include "low_color.h"

#define VERSION "0.1"
#define MAX_COLORS 214

void usage(void);
int read_image(const char* path);
int write_header(const char* filename);
void process_image(void);

ExceptionInfo exception;
char* inputfile;
char* outputfile;
Image *image;
ImageInfo *image_info;
QuantizeInfo quantize_info;

int main(int argc, char** argv)
{
    if(argc<3)
    {
        usage();
        exit(EXIT_FAILURE);
    }

    inputfile=argv[1];
    outputfile=argv[2];

    MagickIncarnate(*argv);
    
    if(!read_image(inputfile))
    {
        fprintf(stderr, "Unable to read image %s - %s\n", inputfile, strerror(errno));
        exit(EXIT_FAILURE);
    }
    
    process_image();
    
    if(!write_header(outputfile))
    {
        fprintf(stderr, "Unable to write header %s - %s\n", outputfile, strerror(errno));
        exit(EXIT_FAILURE);
    }
    
    printf("Generated %s successfully\n\n", outputfile);
    printf("Remember to modify /usr/src/linux/drivers/video/fbcon.c:\n");
    printf(" Change \"#define LOGO_H 80\" to \"#define LOGO_H %i\"\n", image->rows);
    printf(" Change \"#define LOGO_W 80\" to \"#define LOGO_W %i\"\n\n", image->columns);
    printf("Then copy %s to /usr/src/linux/include/linux/linux_logo.h,\n", outputfile);
    printf("and recompile the kernel.\n");

    exit(EXIT_SUCCESS);
}

void usage(void)
{
    printf("Usage:\n");
    printf("fblogo inputfile outputfile\n");
    printf("inputfile is a TIFF file\n");
}

int read_image(const char* path)
{
    GetExceptionInfo(&exception);
    GetQuantizeInfo(&quantize_info);
    image_info=CloneImageInfo((ImageInfo *) NULL);
    (void) strcpy(image_info->filename,path);
    image = ReadImage(image_info,&exception);
    if(image==NULL)
        return 0;
    image->storage_class=DirectClass;
    return 1;
}


void process_image()
{
    quantize_info.number_colors=MAX_COLORS;
    QuantizeImage(&quantize_info, image);
}

int write_header(const char* path)
{
    int i,x,y;
    int fd;
    char buffer[512]={0};
    PixelPacket *pp;
    IndexPacket *ip;

    fd=open(path, O_RDWR|O_CREAT|O_TRUNC, 0644);
    if(fd<0)
        return 0;
    
    for(i=image->colors;i<MAX_COLORS;i++)
    {
        image->colormap[i].red=0;
        image->colormap[i].green=0;
        image->colormap[i].blue=0;
    }
    
    sprintf(buffer, "/* Generated by fblogo version %s */\n\n\0", VERSION);
    write(fd, buffer, strlen(buffer));
    
    write(fd, "#if LINUX_LOGO_COLORS == 214\n\0" ,29);

    sprintf(buffer, "unsigned char linux_logo_red[] __initdata = {\0");
    write(fd, buffer, strlen(buffer));
    for(i=0;i<MAX_COLORS;i++)
    {
        if(i%8==0)
            write(fd, "\n  ", 3);
        sprintf(buffer, "0x%2.2X\0",image->colormap[i].red);
        write(fd, buffer, strlen(buffer));
        if(i!=MAX_COLORS-1)
            write(fd, ", ", 2);
    }
    write(fd,"\n};\n\n", 5);


    sprintf(buffer, "unsigned char linux_logo_green[] __initdata = {\0");
    write(fd, buffer, strlen(buffer));
    for(i=0;i<MAX_COLORS;i++)
    {
        if(i%8==0)
            write(fd, "\n  ", 3);
        sprintf(buffer, "0x%2.2X\0",image->colormap[i].green);
        write(fd, buffer, strlen(buffer));
        if(i!=MAX_COLORS-1)
            write(fd, ", ", 2);
    }
    write(fd,"\n};\n\n", 5);

    sprintf(buffer, "unsigned char linux_logo_blue[] __initdata = {\0");
    write(fd, buffer, strlen(buffer));
    for(i=0;i<MAX_COLORS;i++)
    {
        if(i%8==0)
            write(fd, "\n  ", 3);
        sprintf(buffer, "0x%2.2X\0",image->colormap[i].blue);
        write(fd, buffer, strlen(buffer));
        if(i!=MAX_COLORS-1)
            write(fd, ", ", 2);
    }
    write(fd,"\n};\n\n", 5);

    sprintf(buffer, "unsigned char linux_logo[] __initdata = {\0");
    write(fd, buffer, strlen(buffer));
    ip= GetIndexes(image);

    for (y=0; y < (int) image->rows; y++)
    {
       pp=GetImagePixels(image,0,y,image->columns,1);
       if ((pp == (PixelPacket *) NULL))
          break;
       ip=GetIndexes(image);
       for (x=0; x < (int) image->columns; x++)
       {
          if(x%8==0)
            write(fd, "\n  ", 3);
	  sprintf(buffer, "0x%2.2X\0", ip[x] + 0x20);
          write(fd, buffer, strlen(buffer));
          if((x*y)!=image->columns*image->rows - 1)
            write(fd, ", ", 2);
	}
    }
    write(fd,"\n};\n\n#endif\n\n", 13);

    write(fd, low_color_logo, strlen(low_color_logo));

    close(fd);
    
    return 1;
}
